@@ -42,7 +42,7 @@
 static int fd = -1;
 
 char *pipename = NULL;
-int warned = 0;
+char *default_pipe_name = "/tmp/shairport-sync-audio";
 
 static void start(__attribute__((unused)) int sample_rate,
                   __attribute__((unused)) int sample_format) {
@@ -51,14 +51,18 @@ static void start(__attribute__((unused)) int sample_rate,
   // we check that it's not a "real" error though. From the "man 2 open" page:
   // "ENXIO  O_NONBLOCK | O_WRONLY is set, the named file is a FIFO, and no process has the FIFO
   // open for reading."
-  fd = open(pipename, O_WRONLY | O_NONBLOCK);
-  if ((fd == -1) && (errno != ENXIO) && (warned == 0)) {
+
+  fd = try_to_open_pipe_for_writing(pipename);
+  // we check that it's not a "real" error. From the "man 2 open" page:
+  // "ENXIO  O_NONBLOCK | O_WRONLY is set, the named file is a FIFO, and no process has the FIFO
+  // open for reading." Which is okay.
+  if ((fd == -1) && (errno != ENXIO)) {
     char errorstring[1024];
     strerror_r(errno, (char *)errorstring, sizeof(errorstring));
-    debug(1, "pipe: start -- error %d (\"%s\") opening the pipe named \"%s\".", errno,
+    debug(1, "audio_pipe start -- error %d (\"%s\") opening pipe: \"%s\".", errno,
           (char *)errorstring, pipename);
-    warn("Error %d opening the pipe named \"%s\".", errno, pipename);
-    warned = 1;
+    warn("can not open audio pipe -- error %d (\"%s\") opening pipe: \"%s\".", errno,
+         (char *)errorstring, pipename);
   }
 }
 
@@ -66,32 +70,27 @@ static int play(void *buf, int samples) {
   // if the file is not open, try to open it.
   char errorstring[1024];
   if (fd == -1) {
-    fd = open(pipename, O_WRONLY | O_NONBLOCK);
+    fd = try_to_open_pipe_for_writing(pipename);
   }
   // if it's got a reader, write to it.
   if (fd > 0) {
-    int rc = non_blocking_write(fd, buf, samples * 4);
-    if ((rc < 0) && (warned == 0)) {
+    // int rc = non_blocking_write(fd, buf, samples * 4);
+    int rc = write(fd, buf, samples * 4);
+    if ((rc < 0) && (errno != EPIPE)) {
       strerror_r(errno, (char *)errorstring, 1024);
-      warn("Error %d writing to the pipe named \"%s\": \"%s\".", errno, pipename, errorstring);
-      warned = 1;
+      debug(1, "audio_pip play: error %d writing to the pipe named \"%s\": \"%s\".", errno,
+            pipename, errorstring);
     }
-  } else if ((fd == -1) && (errno != ENXIO) && (warned == 0)) {
-    strerror_r(errno, (char *)errorstring, 1024);
-    warn("Error %d opening the pipe named \"%s\": \"%s\".", errno, pipename, errorstring);
-    warned = 1;
   }
-  return warned;
+  return 0;
 }
 
 static void stop(void) {
   // Don't close the pipe just because a play session has stopped.
-  //  if (fd > 0)
-  //    close(fd);
 }
 
 static int init(int argc, char **argv) {
-  debug(1, "pipe init");
+  //  debug(1, "pipe init");
   //  const char *str;
   //  int value;
   //  double dvalue;
@@ -110,22 +109,27 @@ static int init(int argc, char **argv) {
     if (config_lookup_string(config.cfg, "pipe.name", &str)) {
       pipename = (char *)str;
     }
-
-    if ((pipename) && (strcasecmp(pipename, "STDOUT") == 0))
-      die("Can't use \"pipe\" backend for STDOUT. Use the \"stdout\" backend instead.");
   }
 
-  if ((pipename == NULL) && (argc != 1))
-    die("bad or missing argument(s) to pipe");
+  if (argc > 1)
+    die("too many command-line arguments to pipe");
 
   if (argc == 1)
-    pipename = strdup(argv[0]);
+    pipename = argv[0]; // command line argument has priority
+
+  if ((pipename) && (strcasecmp(pipename, "STDOUT") == 0))
+    die("Can't use \"pipe\" backend for STDOUT. Use the \"stdout\" backend instead.");
+
+  if (pipename == NULL)
+    pipename = default_pipe_name; // if none specified
 
   // here, create the pipe
-  if (mkfifo(pipename, 0644) && errno != EEXIST)
-    die("Could not create output pipe \"%s\"", pipename);
+  mode_t oldumask = umask(000);
+  if (mkfifo(pipename, 0666) && errno != EEXIST)
+    die("Could not create audio pipe \"%s\"", pipename);
+  umask(oldumask);
 
-  debug(1, "Pipename is \"%s\"", pipename);
+  debug(1, "audio pipe name is \"%s\"", pipename);
 
   return 0;
 }
