@@ -7,9 +7,6 @@
 
 #include <soundio/soundio.h>
 
-int Fs;
-long long starttime, samples_played;
-
 struct SoundIoOutStream *outstream;
 struct SoundIo *soundio;
 struct SoundIoDevice *device;
@@ -27,8 +24,9 @@ static void write_callback(struct SoundIoOutStream *outstream, int frame_count_m
   int fill_bytes = soundio_ring_buffer_fill_count(ring_buffer);
   int fill_count = fill_bytes / outstream->bytes_per_frame;
 
-  debug(3, "[--->>] frame_count_min: %d , frame_count_max: %d , fill_bytes: %d , fill_count: %d , "
-           "outstream->bytes_per_frame: %d",
+  debug(3,
+        "[--->>] frame_count_min: %d , frame_count_max: %d , fill_bytes: %d , fill_count: %d , "
+        "outstream->bytes_per_frame: %d",
         frame_count_min, frame_count_max, fill_bytes, fill_count, outstream->bytes_per_frame);
 
   if (frame_count_min > fill_count) {
@@ -129,9 +127,6 @@ static void deinit(void) {
 }
 
 static void start(int sample_rate, int sample_format) {
-  Fs = sample_rate;
-  starttime = 0;
-  samples_played = 0;
   int err;
 
   debug(1, "soundion rate: %d, format: %d", sample_rate, sample_format);
