@@ -2,7 +2,7 @@
 #define _MDNS_H
 
 #include "config.h"
-#include <player.h>
+#include "player.h"
 #include <stdint.h>
 
 extern int mdns_pid;
